#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Application entry point."""

# yapf: disable

# Standard library imports
import os
import shutil
import sys
import logging

# Local imports
from anaconda_navigator import __version__
from anaconda_navigator.app.cli import parse_arguments
from anaconda_navigator.exceptions import exception_handler
from anaconda_navigator.utils.conda import is_conda_available


# yapf: enable

bis_debug = False


def is_debug_enabled():
    return bis_debug


def set_debug(flag=True):
    global bis_debug
    bis_debug = flag


def main():  # pragma: no cover
    """Main application entry point."""
    global bis_debug
    # Check if conda is available
    if not is_conda_available():
        path = os.path.abspath(os.path.dirname(sys.argv[0]))
        # print(path, len(sys.argv))
        msg = '''#
# Please activate the conda root enviroment properly before running the
# `anaconda-navigator` command.
'''
        win_msg = '''#
# To activate the environment please open a Windows Command Prompt and run:
#
#   {0}\\activate root
'''.format(path)

        unix_msg = '''#
# To activate the environment please open a terminal and run:
#
#   . {0}/activate root
'''.format(path)

        more_info = '''#
# For more information please see the documentation at:
#
#   https://docs.anaconda.com/anaconda/navigator/
#'''
        if os.name == 'nt':
            print_msg = '{}{}{}'.format(msg, win_msg, more_info)
        else:
            print_msg = '{}{}{}'.format(msg, unix_msg, more_info)

        print(print_msg)

        return 1

    # Parse CLI arguments
    options = parse_arguments()

    # Return information on version
    if options.version:
        print(__version__)
        sys.exit(0)

    if options.log_level == logging.DEBUG:
        bis_debug = True

    # Reset Navigator conifg
    if options.reset:
        print('\nAnaconda Navigator configuration reset...\n\n')
        from anaconda_navigator.config import CONF_PATH
        if os.path.isdir(CONF_PATH):
            try:
                shutil.rmtree(CONF_PATH)
                print('Anaconda Navigator configuration reset successful!\n')
                sys.exit(0)
            except Exception as e:
                print('Anaconda Navigator configuration reset failed!!!\n')
                print(e)
                sys.exit(1)

    if options.removelock:
        print('\nRemoving Anaconda Navigator lock...\n\n')
        from anaconda_navigator.utils.misc import remove_lock, remove_pid
        lock = remove_lock()
        pid = remove_pid()
        if lock and pid:
            print('Anaconda Navigator lock removal successful!\n')
            sys.exit(0)
        else:
            print('Anaconda Navigator lock removal failed!!!\n')
            sys.exit(1)

    # Clean old style logs
    from anaconda_navigator.utils.logs import clean_logs
    clean_logs()

    # Import app
    from anaconda_navigator.app.start import start_app
    return exception_handler(start_app, options)


if __name__ == '__main__':  # pragma: no cover
    main()



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Worker threads for using the anaconda-client api."""

# Standard library imports
from collections import deque
from traceback import format_exc
import json
import logging
import re
import time

# Third party imports
import binstar_client
import requests
from binstar_client.errors import Unauthorized
from requests.exceptions import SSLError
from qtpy.QtCore import QObject, QThread, QTimer, Signal

# Local imports
from anaconda_navigator.api.conda_api import CondaAPI
from anaconda_navigator.api.team_edition_api import TeamEditionAPI
from anaconda_navigator.config import CONF, DEFAULT_BRAND
from anaconda_navigator.utils import constants as C
from anaconda_navigator.utils import sort_versions
from anaconda_navigator.utils.logs import logger
from anaconda_navigator.utils.py3compat import is_text_string, to_text_string
from anaconda_navigator.widgets.dialogs import MessageBoxError


class ClientWorker(QObject):
    """Anaconda Client API process worker."""

    sig_chain_finished = Signal(object, object, object)
    sig_finished = Signal(object, object, object)

    def __init__(self, method, args, kwargs):
        """Anaconda Client API process worker."""
        super(ClientWorker, self).__init__()
        self.method = method
        self.args = args
        self.kwargs = kwargs
        self._is_finished = False

    def is_finished(self):
        """Return wether or not the worker has finished running the task."""
        return self._is_finished

    def start(self):
        """Start the worker process."""
        error, output = None, None
        try:
            time.sleep(0.1)
            output = self.method(*self.args, **self.kwargs)
        except Exception as err:
            logger.debug(str((self.method.__module__, self.method.__name__, err)), )
            error = str(err)
            error = error.replace('(', '')
            error = error.replace(')', '')

        self.sig_finished.emit(self, output, error)
        self._is_finished = True


class Args:
    """Dummy class to pass to anaconda client on token loading and removal."""


class _ClientAPI(QObject):
    """Anaconda Client API wrapper."""

    DEFAULT_TIMEOUT = 6

    def __init__(self, config=None):
        """Anaconda Client API wrapper."""
        super(QObject, self).__init__()
        self._conda_api = CondaAPI()
        self._anaconda_client_api = None
        self._config = config
        self._queue = deque()
        self._threads = []
        self._workers = []
        self._timer = QTimer()
        self.config = CONF

        self._timer.setInterval(1000)
        self._timer.timeout.connect(self._clean)

        # Setup
        self.reload_client()

    @property
    def anaconda_client_api(self):
        return self._anaconda_client_api

    def _clean(self):
        """Check for inactive workers and remove their references."""
        if self._workers:
            for w in self._workers:
                if w.is_finished():
                    self._workers.remove(w)

        if self._threads:
            for t in self._threads:
                if t.isFinished():
                    self._threads.remove(t)
        else:
            self._timer.stop()

    def _start(self):
        """Take avalaible worker from the queue and start it."""
        if len(self._queue) == 1:
            thread = self._queue.popleft()
            thread.start()
            self._timer.start()

    def _create_worker(self, method, *args, **kwargs):
        """Create a worker for this client to be run in a separate thread."""
        # FIXME: this might be heavy...
        thread = QThread()
        worker = ClientWorker(method, args, kwargs)
        worker.moveToThread(thread)
        worker.sig_finished.connect(self._start)
        worker.sig_finished.connect(thread.quit)
        thread.started.connect(worker.start)
        self._queue.append(thread)
        self._threads.append(thread)
        self._workers.append(worker)
        self._start()
        return worker

    def _is_internet_available(self):
        """Check initernet availability."""
        if self._config:
            config_value = self._config.get('main', 'offline_mode')
        else:
            config_value = False

        if config_value:
            connectivity = False
        else:
            connectivity = True  # is_internet_available()

        return connectivity

    # --- Callbacks
    # -------------------------------------------------------------------------
    @staticmethod
    def _load_repodata(repodata, metadata=None, python_version=None):
        """
        Load all the available package information.

        See load_repadata for full documentation.
        """
        metadata = metadata if metadata else {}

        all_packages = {}
        for channel_url, data in repodata.items():
            packages = data.get('packages', {})

            for canonical_name in packages:
                data = packages[canonical_name]
                # Do not filter based on python version
                # if (python_version and not is_dependency_met(
                #         data['depends'], python_version, 'python')):
                #     continue
                name, version, b = tuple(canonical_name.rsplit('-', 2))

                if name not in all_packages:
                    all_packages[name] = {
                        'versions': set(),
                        'size': {},
                        'type': {},
                        'app_entry': {},
                        'app_type': {},
                    }
                elif name in metadata:
                    temp_data = all_packages[name]
                    temp_data['home'] = metadata[name].get('home', '')
                    temp_data['license'] = metadata[name].get('license', '')
                    temp_data['summary'] = metadata[name].get('summary', '')
                    temp_data['latest_version'] = metadata[name].get('version')
                    all_packages[name] = temp_data

                all_packages[name]['versions'].add(version)
                all_packages[name]['size'][version] = data.get('size', '')

                # Only the latest builds will have the correct metadata for
                # apps, so only store apps that have the app metadata
                if data.get('type'):
                    all_packages[name]['type'][version] = data.get('type')
                    all_packages[name]['app_entry'][version] = data.get('app_entry')
                    all_packages[name]['app_type'][version] = data.get('app_type')

        # Calculate the correct latest_version
        for name in all_packages:
            versions = tuple(sorted(all_packages[name]['versions'], reverse=True))
            all_packages[name]['latest_version'] = versions[0]

        all_apps = {}
        for name in all_packages:
            versions = sort_versions(list(all_packages[name]['versions']))
            all_packages[name]['versions'] = versions[:]

            for version in versions:
                has_type = all_packages[name].get('type')
                # Has type in this case implies being an app
                if has_type:
                    all_apps[name] = all_packages[name].copy()
                    # Remove all versions that are not apps!
                    versions = all_apps[name]['versions'][:]
                    types = all_apps[name]['type']
                    app_versions = [v for v in versions if v in types]
                    all_apps[name]['versions'] = app_versions
        return all_packages, all_apps

    @staticmethod
    def _prepare_model_data(packages, linked, pip=None, metadata=None):
        """Prepare model data for the packages table model."""
        pip = pip if pip else []
        data = []
        linked_packages = {}
        for canonical_name in linked:
            name, version, b = tuple(canonical_name.rsplit('-', 2))
            linked_packages[name] = {'version': version}

        pip_packages = {}
        for canonical_name in pip:
            name, version, b = tuple(canonical_name.rsplit('-', 2))
            pip_packages[name] = {'version': version}

        packages_names = sorted(
            list(set(list(linked_packages.keys()) + list(pip_packages.keys()) + list(packages.keys())), )
        )
        packages_metadata = metadata or dict()
        for name in packages_names:
            p_data = packages.get(name) or packages_metadata.get(name, dict())

            summary = p_data.get('summary') or ''
            url = p_data.get('home') or ''
            license_ = p_data.get('license') or ''
            versions = p_data.get('versions') or []
            version = p_data.get('latest_version') or ''

            if name in pip_packages:
                type_ = C.PIP_PACKAGE
                version = pip_packages[name].get('version', '')
                status = C.INSTALLED
            elif name in linked_packages:
                type_ = C.CONDA_PACKAGE
                version = linked_packages[name].get('version', '')
                status = C.INSTALLED

                if version in versions:
                    vers = versions
                    upgradable = not version == vers[-1] and len(vers) != 1
                    downgradable = not version == vers[0] and len(vers) != 1

                    if upgradable and downgradable:
                        status = C.MIXGRADABLE
                    elif upgradable:
                        status = C.UPGRADABLE
                    elif downgradable:
                        status = C.DOWNGRADABLE
            else:
                type_ = C.CONDA_PACKAGE
                status = C.NOT_INSTALLED

            row = {
                C.COL_ACTION: C.ACTION_NONE,
                C.COL_PACKAGE_TYPE: type_,
                C.COL_NAME: name,
                C.COL_DESCRIPTION: summary.capitalize(),
                C.COL_VERSION: version,
                C.COL_STATUS: status,
                C.COL_URL: url,
                C.COL_LICENSE: license_,
                C.COL_ACTION_VERSION: None,
            }

            data.append(row)
        return data

    def _get_user_licenses(self, products=None):
        """Get user trial/paid licenses from anaconda.org."""
        license_data = []
        try:
            res = self._anaconda_client_api.user_licenses()
            license_data = res.get('data', [])

            # This should be returning a dict or list not a json string!
            if is_text_string(license_data):
                license_data = json.loads(license_data)
        except Exception:
            time.sleep(0.3)

        return license_data

    # --- Public API
    # -------------------------------------------------------------------------
    def reload_client(self):
        """
        Sets the client depending on the settings from the configuration file.
        """
        logged_api_url = self.config.get('main', 'logged_api_url', None)
        anaconda_api_url = self.config.get('main', 'anaconda_api_url', None)
        team_edition_api_url = self.config.get('main', 'team_edition_api_url', None)
        enterprise_4_repo_api_url = self.config.get('main', 'enterprise_4_repo_api_url', None)

        if logged_api_url and logged_api_url in [anaconda_api_url, enterprise_4_repo_api_url]:
            return self._load_binstar_client(logged_api_url)
        elif logged_api_url and logged_api_url == team_edition_api_url:
            return self._load_team_edition_client(logged_api_url)
        else:
            # Looks like there wasn't any action to login from the Navigator application.
            # Checking if there was a login action into Binstar client through the CLI.
            url = binstar_client.utils.get_config()['url']

            try:
                client = self._load_binstar_client(url)
                client.user()
                return client

            except (Unauthorized, SSLError) as e:
                logger.debug(str(e))

                # No users authorized through banister client.
                # Return Binstar client with default Anaconda API url.
                return self._load_binstar_client(anaconda_api_url)

            except requests.exceptions.ConnectionError:
                # No connection so we are using default client.
                self._load_binstar_client(anaconda_api_url)

    def _load_binstar_client(self, api_url):
        """
        Recreate the binstar client with new updated values.

        Notes:
        ------
        The Client needs to be restarted because on domain change it will not
        validate the user since it will check against the old domain, which
        was used to create the original client.

        See: https://github.com/ContinuumIO/navigator/issues/1325
        """
        config = binstar_client.utils.get_config()
        config['url'] = api_url
        for site in config['sites'].values():
            site['url'] = api_url

        binstar_client.utils.set_config(config)

        token = self.load_token()
        binstar = binstar_client.utils.get_server_api(
            token=token, site=None, cls=None, config=config, log_level=logging.NOTSET
        )
        self._anaconda_client_api = binstar
        return binstar

    def _load_team_edition_client(self, api_url):
        """
        Sets the '_anaconda_client_api' to user TeamEdition API insted of
        default Binstar client.
        """
        verify_ssl = self._conda_api.load_rc().get('ssl_verify', False)
        self._anaconda_client_api = TeamEditionAPI(api_url, self.config, verify_ssl)
        return self._anaconda_client_api

    def token(self):
        """Return the current token registered with authenticate."""
        return self._anaconda_client_api.token

    def load_token(self):
        """Load current authenticated token."""
        token = None
        try:
            if isinstance(self._anaconda_client_api, TeamEditionAPI):
                token = self._anaconda_client_api.load_token()
            else:
                token = binstar_client.utils.load_token(self.get_api_url())
        except Exception:
            pass

        return token

    def _login(self, username, password, application, application_url):
        """Login callback."""
        if isinstance(self._anaconda_client_api, TeamEditionAPI):
            new_token = self._anaconda_client_api.authenticate(username, password)
            self._anaconda_client_api.store_token(new_token)

        else:
            new_token = self._anaconda_client_api.authenticate(username, password, application, application_url)

            args = Args()
            args.site = None
            args.token = new_token

            binstar_client.utils.store_token(new_token, args)

        return new_token

    def login(self, username, password, application, application_url):
        """Login to anaconda server."""
        logger.debug(str((username, application, application_url)))
        method = self._login
        return self._create_worker(method, username, password, application, application_url)

    def logout(self):
        """
        Logout from anaconda.org.

        This method removes the authentication and removes the token.
        """
        error = None
        args = Args()
        args.site = None
        args.token = self.token

        if isinstance(self._anaconda_client_api, TeamEditionAPI):
            self._anaconda_client_api.logout()
        else:
            binstar_client.utils.remove_token(args)

            if self.token:
                try:
                    self._anaconda_client_api.remove_authentication()
                except binstar_client.errors.Unauthorized as e:
                    error = e
                    logger.debug("The token that you are trying to remove may " "not be valid {}".format(e))
                except Exception as e:
                    error = e
                    logger.debug("The certificate might be invalid. {}".format(e))

            logger.info("logout successful")

        self.config.reset_logged_data()
        return error

    def load_repodata(self, repodata, metadata=None, python_version=None):
        """
        Load all the available packages information for downloaded repodata.

        For downloaded repodata files (repo.anaconda.com), additional
        data provided (anaconda cloud), and additional metadata and merge into
        a single set of packages and apps.

        If python_version is not none, exclude all package/versions which
        require an incompatible version of python.

        Parameters
        ----------
        repodata: dict of dicts
            Data loaded from the conda cache directories.
        metadata: dict
            Metadata info form different sources. For now only from
            repo.anaconda.com
        python_version: str
            Python version used in preprocessing.
        """
        logger.debug('')
        method = self._load_repodata
        return self._create_worker(
            method,
            repodata,
            metadata=metadata,
            python_version=python_version,
        )

    def prepare_model_data(self, packages, linked, pip=None, metadata=None):
        """Prepare downloaded package info along with pip pacakges info."""
        logger.debug('')
        method = self._prepare_model_data
        return self._create_worker(
            method,
            packages,
            linked,
            pip=pip,
            metadata=metadata
        )

    def user(self):
        """Return current logged user information."""
        return self.organizations(login=None)

    def domain(self):
        """Return current domain."""
        return self._anaconda_client_api.domain

    def packages(self, login=None, platform=None, package_type=None, type_=None, access=None):
        """Return all the available packages for a given user.

        Parameters
        ----------
        type_: Optional[str]
            Only find packages that have this conda `type`, (i.e. 'app').
        access : Optional[str]
            Only find packages that have this access level (e.g. 'private',
            'authenticated', 'public').
        """
        logger.debug('')
        method = self._anaconda_client_api.user_packages
        return self._create_worker(
            method,
            login=login,
            platform=platform,
            package_type=package_type,
            type_=type_,
            access=access,
        )

    def organizations(self, login):
        """List all the organizations a user has access to."""
        try:
            user = self._anaconda_client_api.user(login=login)
        except Exception:
            user = {}
        return user

    def get_api_url(self):
        """Get the anaconda client url configuration."""
        if isinstance(self._anaconda_client_api, TeamEditionAPI):
            return self.config.get('main', 'team_edition_api_url')
        else:
            config_data = binstar_client.utils.get_config()
            return config_data.get('url', 'https://api.anaconda.org')

    def get_api_info_url(self):
        """Get the anaconda client info url configuration."""
        if isinstance(self._anaconda_client_api, TeamEditionAPI):
            return "{0}/api/system".format(self.config.get('main', 'team_edition_api_url'))
        else:
            config_data = binstar_client.utils.get_config()
            return config_data.get('url', 'https://api.anaconda.org')

    @staticmethod
    def set_api_url(url):
        """Set the anaconda client url configuration."""
        config_data = binstar_client.utils.get_config()
        config_data['url'] = url
        try:
            binstar_client.utils.set_config(config_data)
        except Exception as e:
            logger.error('Could not write anaconda client configuration')
            msg_box = MessageBoxError(
                title='Anaconda Client configuration error',
                text='Anaconda Client configuration could not be updated.<br>'
                'This may result in Navigator not working properly.<br>',
                error=e,
                report=False,
                learn_more=None,
            )
            msg_box.exec_()

    def get_ssl(self, set_conda_ssl=True):
        """
        Get conda ssl configuration and set navigator and anaconda-client accordingly.
        """
        config = binstar_client.utils.get_config()
        value = config.get('ssl_verify', config.get('ssl_verify', True))

        if set_conda_ssl:
            ssl_verify = self._conda_api.config_get('ssl_verify').communicate()[0].get('ssl_verify')
            config['ssl_verify'] = ssl_verify
            config['verify_ssl'] = ssl_verify
            binstar_client.utils.set_config(config)

            if isinstance(ssl_verify, bool):
                self.config.set('main', 'ssl_verification', ssl_verify)
                self.config.set('main', 'ssl_certificate', None)
            else:
                self.config.set('main', 'ssl_verification', True)
                self.config.set('main', 'ssl_certificate', ssl_verify)

            return ssl_verify

        return value

    def set_ssl(self, value):
        """Set the anaconda client url configuration."""
        config_data = binstar_client.utils.get_config()
        config_data['verify_ssl'] = value
        config_data['ssl_verify'] = value
        try:
            binstar_client.utils.set_config(config_data)
            self._conda_api.config_set('ssl_verify', value).communicate()
        except Exception as e:
            logger.error('Could not write anaconda client configuration')
            msg_box = MessageBoxError(
                title='Anaconda Client configuration error',
                text='Anaconda Client configuration could not be updated.<br>'
                'This may result in Navigator not working properly.<br>',
                error=e,
                report=False,
                learn_more=None,
            )
            msg_box.exec_()

    def get_user_licenses(self, products=None):
        """Get user trial/paid licenses from anaconda.org."""
        logger.debug(str((products)))
        method = self._get_user_licenses
        return self._create_worker(method, products=products)

    def _get_api_info(self, url, proxy_servers=None, verify=True):
        """Callback."""
        proxy_servers = proxy_servers or {}
        data = {
            "api_url": url,
            "api_docs_url": "https://api.anaconda.org/docs",
            "brand": DEFAULT_BRAND,
            "conda_url": "https://conda.anaconda.org",
            "main_url": "https://anaconda.org",
            "pypi_url": "https://pypi.anaconda.org",
            "swagger_url": "https://api.anaconda.org/swagger.json",
        }
        if self._is_internet_available():
            try:
                r = requests.get(
                    url,
                    proxies=proxy_servers,
                    verify=verify,
                    timeout=self.DEFAULT_TIMEOUT,
                )
                content = to_text_string(r.content, encoding='utf-8')
                new_data = json.loads(content)

                # Enforce no trailing slash
                for key, value in new_data.items():
                    if is_text_string(value):
                        data[key] = value[:-1] if value[-1] == '/' else value

            except Exception as error:
                logger.error(str(error))

        return data

    def get_api_info(self, url, proxy_servers=None, verify=True):
        """Query anaconda api info."""
        logger.debug(str((url)))
        proxy_servers = proxy_servers or {}
        method = self._get_api_info
        return self._create_worker(method, url, proxy_servers=proxy_servers, verify=verify)


CLIENT_API = None


def ClientAPI(config=None):
    """Client API threaded worker."""
    global CLIENT_API

    if CLIENT_API is None:
        CLIENT_API = _ClientAPI(config=config)

    return CLIENT_API


def local_test():  # pragma: no cover
    """Local main test."""
    from anaconda_navigator.utils.qthelpers import qapplication

    app = qapplication()
    api = ClientAPI()
    api.login('goanpeca', 'asdasd', 'baby', '')
    api.login('bruce', 'asdasd', 'baby', '')

    app.exec_()


if __name__ == '__main__':  # pragma: no cover
    local_test()



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Dialogs for backing up environments."""

__all__ = ['BackupSelectorDialog']

import html
import os
import tempfile
import typing
import uuid
import requests
from qtpy import compat
from qtpy import QtWidgets
from anaconda_navigator.api import nucleus
from anaconda_navigator.api.nucleus import tools as nucleus_tools
from . import multiaccount_dialogs

if typing.TYPE_CHECKING:
    import typing_extensions
    from anaconda_navigator.widgets import main_window


DEFAULT_ERROR: 'typing_extensions.Final[str]' = 'Unable to backup environment due to an unknown error.'


class BackupSelectorDialog(multiaccount_dialogs.PrepopulatedSelectorDialog):
    """Dialog for selecting the target to which to export an environment."""

    def __init__(self, parent: 'main_window.MainWindow') -> None:
        """Initialize new :class:`~BackupSelectorDialog` instance."""
        super().__init__(parent=parent)

        self.setWindowTitle('Backup Environment')

    def __init_header__(
            self,
            layout: QtWidgets.QVBoxLayout,
            *,
            caption_text: str = 'Select location to backup environment:',
    ) -> None:
        """Initialize header part of the dialog."""
        super().__init_header__(layout, caption_text=caption_text)

    def __init_footer__(
            self,
            layout: QtWidgets.QVBoxLayout,
            *,
            caption_text: str = 'Backup as:',
    ) -> None:
        """Initialize footer part of the dialog."""
        super().__init_footer__(layout, caption_text=caption_text)
        self._controls.account_nucleus += self._controls.all[-3:]

    def __init_actions__(
            self,
            layout: QtWidgets.QVBoxLayout,
            *,
            accept_text: str = 'Backup',
            reject_text: str = 'Cancel',
    ) -> None:
        """Initialize actions part of the dialog."""
        super().__init_actions__(layout, accept_text=accept_text, reject_text=reject_text)

    def _process_accept(self) -> None:
        """Process clicking on the 'OK' button."""
        self.set_busy(True)
        self.clear_heading_errors()
        self.footer_error = ''

        if self.selection == multiaccount_dialogs.SelectorValue.LOCAL:
            self.__process_accept_local()
        elif self.selection == multiaccount_dialogs.SelectorValue.NUCLEUS:
            self.__process_accept_nucleus()
        else:
            raise NotImplementedError()

    def _process_selection(self, value: multiaccount_dialogs.SelectorValue) -> None:
        """Process changing selected value in the dialog."""
        super()._process_selection(value)
        self.__update_acceptable()

    def _process_environment_name(self, value: str) -> None:
        """Process change of the `environment_name` value."""
        super()._process_environment_name(value)
        self.__update_acceptable()

    def __update_acceptable(self) -> None:
        """Update state of the accept button according to current dialog state."""
        if self.selection == multiaccount_dialogs.SelectorValue.LOCAL:
            self.set_acceptable(True)
            return

        if (self.selection == multiaccount_dialogs.SelectorValue.NUCLEUS) and self.environment_name:
            self.set_acceptable(True)
            return

        self.set_acceptable(False)

    # Local

    def __process_accept_local(self) -> None:
        """Process clicking on the 'OK' button when local account is selected."""
        filter_yaml: 'typing_extensions.Final[str]' = 'Conda environment files (*.yaml *.yml)'
        filter_any: 'typing_extensions.Final[str]' = 'All files (*)'

        path: str
        selected_filter: str
        path, selected_filter = compat.getsavefilename(
            parent=self,
            caption='Backup Environment',
            basedir=os.path.expanduser('~'),
            filters=';;'.join([filter_yaml, filter_any]),
        )
        if not path:
            self.set_busy(False)
            return
        if (not os.path.splitext(path)[1]) and (selected_filter == filter_yaml):
            path += '.yaml'

        worker = self.parent().api.export_environment(
            prefix=self.parent().current_prefix,
            file=os.path.join(tempfile.gettempdir(), uuid.uuid4().hex + '.yml'),
        )
        worker.requested_file = path
        worker.sig_finished.connect(self.__check_local_export)

    def __check_local_export(self, worker: typing.Any, output: typing.Any, error: typing.Any) -> None:
        """Check result of :code:`conda env export` command."""
        if not self.isVisible():
            try:
                os.remove(worker.file)
            except OSError:
                pass
            return

        if not os.path.isfile(worker.file):
            self.set_busy(False)
            self.add_heading_error(multiaccount_dialogs.HEADING_ERROR_TEMPLATE.format(
                content='Unable to create requested file.',
            ))
            return

        new_worker: nucleus.TaskWorker = nucleus_tools.clear_environment.worker(
            source_file=worker.file,
            target_file=worker.requested_file,
            name=worker.name,
        )
        self.finished.connect(new_worker.cancel)
        new_worker.signals.sig_done.connect(lambda result: self.finished.disconnect(new_worker.cancel))
        new_worker.signals.sig_done.connect(self.__check_local_inject)
        new_worker.start()

    def __check_local_inject(self, result: nucleus.TaskResult) -> None:
        """"""
        try:
            os.remove(result.call.kwargs['source_file'])
        except OSError:
            pass

        if result.status == nucleus.TaskStatus.SUCCEEDED:
            super()._process_accept()
            return

        try:
            os.remove(result.call.kwargs['target_file'])
        except OSError:
            pass

        if result.status == nucleus.TaskStatus.FAILED:
            self.add_heading_error(multiaccount_dialogs.HEADING_ERROR_TEMPLATE.format(
                content='Unable to create requested file.',
            ))

        self.set_busy(False)

    # Nucleus

    def __process_accept_nucleus(self) -> None:
        """Process clicking on the 'OK' button when Nucleus account is selected."""
        worker = self.parent().api.export_environment(
            prefix=self.parent().current_prefix,
            file=os.path.join(tempfile.gettempdir(), uuid.uuid4().hex + '.yml'),
        )
        worker.sig_finished.connect(self.__check_nucleus_export)

    def __check_nucleus_export(self, worker: typing.Any, output: typing.Any, error: typing.Any) -> None:
        """Check result of :code:`conda env export` command."""
        if not self.isVisible():
            try:
                os.remove(worker.file)
            except OSError:
                pass
            return

        if not os.path.isfile(worker.file):
            self.set_busy(False)
            self.add_heading_error(multiaccount_dialogs.HEADING_ERROR_TEMPLATE.format(
                content='System error creating a backup.',
            ))
            return

        new_worker: nucleus.TaskWorker = nucleus_tools.clear_environment.worker(
            source_file=worker.file,
            target_file=os.path.join(tempfile.gettempdir(), uuid.uuid4().hex + '.yml'),
            name=worker.name,
        )
        self.finished.connect(new_worker.cancel)
        new_worker.signals.sig_done.connect(lambda result: self.finished.disconnect(new_worker.cancel))
        new_worker.signals.sig_done.connect(self.__check_nucleus_inject)
        new_worker.start()

    def __check_nucleus_inject(self, result: nucleus.TaskResult) -> None:
        """"""
        try:
            os.remove(result.call.kwargs['source_file'])
        except OSError:
            pass

        if result.status == nucleus.TaskStatus.SUCCEEDED:
            new_worker: nucleus.TaskWorker = nucleus.NucleusAPI().create_environment.worker(
                name=self.environment_name,
                path=result.call.kwargs['target_file'],
            )
            self.finished.connect(new_worker.cancel)
            new_worker.signals.sig_done.connect(lambda result: self.finished.disconnect(new_worker.cancel))
            new_worker.signals.sig_done.connect(self.__check_nucleus_create)
            new_worker.start()
            return

        try:
            os.remove(result.call.kwargs['target_file'])
        except OSError:
            pass

        if result.status == nucleus.TaskStatus.FAILED:
            self.add_heading_error(multiaccount_dialogs.HEADING_ERROR_TEMPLATE.format(
                content='Unable to create requested file.',
            ))

        self.set_busy(False)

    def __check_nucleus_create(self, result: nucleus.TaskResult) -> None:
        """Check Nucleus response after creating a new environment."""
        if self.isVisible() and (result.status == nucleus.TaskStatus.FAILED):
            handlers: nucleus_tools.HttpErrorHandlers = nucleus_tools.HttpErrorHandlers()
            handlers.register_handler(BaseException, self._handle_header_error(DEFAULT_ERROR))
            handlers.register_handler(nucleus.LoginRequiredException, self._handle_nucleus_login_required())
            handlers.register_http_handler(
                self._handle_environment_already_exists(path=result.call.kwargs['path']),
                409,
                'environment_already_exists',
            )
            handlers.register_http_handler(self._handle_unprocessable_entry(), 422)

            if handlers.handle(exception=typing.cast(BaseException, result.exception)):
                return

            self.set_busy(False)

        try:
            os.remove(result.call.kwargs['path'])
        except OSError:
            pass

        if self.isVisible() and (result.status == nucleus.TaskStatus.SUCCEEDED):
            super()._process_accept()

    def __check_nucleus_update(self, result: nucleus.TaskResult) -> None:
        """Check Nucleus response after updating existing environment."""
        try:
            os.remove(result.call.kwargs['path'])
        except OSError:
            pass

        if not self.isVisible():
            return

        if result.status == nucleus.TaskStatus.SUCCEEDED:
            super()._process_accept()

        elif result.status == nucleus.TaskStatus.FAILED:
            handlers: nucleus_tools.HttpErrorHandlers = nucleus_tools.HttpErrorHandlers()
            handlers.register_handler(BaseException, self._handle_header_error(DEFAULT_ERROR))
            handlers.register_handler(nucleus.LoginRequiredException, self._handle_nucleus_login_required)
            handlers.handle(exception=typing.cast(BaseException, result.exception))

        self.set_busy(False)

    # handlers

    def _handle_environment_already_exists(self, path: str) -> typing.Callable[['requests.RequestException'], bool]:
        """
        Handle conflict on creating a new environment in Nucleus.

        If `environment_override` is set - may launch update of existing Nucleus environment.
        """
        def result(exception: BaseException) -> bool:
            if self.environment_override:
                worker: nucleus.TaskWorker = nucleus.NucleusAPI().update_environment.worker(
                    name=self.environment_name,
                    path=path,
                )
                self.finished.connect(worker.cancel)
                worker.signals.sig_done.connect(lambda result: self.finished.disconnect(worker.cancel))
                worker.signals.sig_done.connect(self.__check_nucleus_update)
                worker.start()
                return True

            self.footer_error = multiaccount_dialogs.FOOTER_ERROR_TEMPLATE.format(
                content='Please rename environment to continue.',
            )
            return False

        return result

    def _handle_unprocessable_entry(self) -> typing.Callable[['requests.RequestException'], bool]:
        """Handle 422 HTTP error."""
        def result(exception: requests.RequestException) -> bool:
            items: typing.List[typing.Any] = []
            if exception.response is not None:
                try:
                    items.extend(exception.response.json()['detail'])
                except (ValueError, TypeError, KeyError):
                    pass

            pending: bool = True
            for item in items:
                try:
                    if item['loc'] == ['body', 'name']:
                        self.footer_error = multiaccount_dialogs.FOOTER_ERROR_TEMPLATE.format(
                            content=html.escape(item['msg']),
                        )
                        pending = False
                except (TypeError, KeyError):
                    pass

            if pending:
                self.add_heading_error(multiaccount_dialogs.HEADING_ERROR_TEMPLATE.format(content=DEFAULT_ERROR))
            return False

        return result



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2013-2017 Continuum Analytics, Inc.
# Copyright (c) 2010 Preston Landers (Released under the Python 2.6.5 license)
# Copyright (c) 2008-2011 by Enthought, Inc.
# All rights reserved.
#
# Licensed under the terms of the BSD 3-clause License (See LICENSE.txt)
# -----------------------------------------------------------------------------
"""
See: http://stackoverflow.com/a/19719292/1170370 on 20160407 MCS.
"""

# yapf: disable

from __future__ import print_function, unicode_literals

# Standard library imports
import ctypes
import enum
import os
import shlex
import sys
import traceback


# yapf: enable


def is_user_admin():
    if os.name == 'nt':
        try:
            return ctypes.windll.shell32.IsUserAnAdmin()
        except Exception:
            traceback.print_exc()
            return False
    elif os.name == 'posix':
        # Check for root on Posix
        return os.getuid() == 0
    else:
        raise RuntimeError("Unsupported operating system for this module: %s" % (os.name, ))


class SW(enum.IntEnum):
    HIDE = 0
    MAXIMIZE = 3
    MINIMIZE = 6
    RESTORE = 9
    SHOW = 5
    SHOWDEFAULT = 10
    SHOWMAXIMIZED = 3
    SHOWMINIMIZED = 2
    SHOWMINNOACTIVE = 7
    SHOWNA = 8
    SHOWNOACTIVATE = 4
    SHOWNORMAL = 1


class ERROR(enum.IntEnum):
    ZERO = 0
    FILE_NOT_FOUND = 2
    PATH_NOT_FOUND = 3
    BAD_FORMAT = 11
    ACCESS_DENIED = 5
    ASSOC_INCOMPLETE = 27
    DDE_BUSY = 30
    DDE_FAIL = 29
    DDE_TIMEOUT = 28
    DLL_NOT_FOUND = 32
    NO_ASSOC = 31
    OOM = 8
    SHARE = 26


def run_as_admin(cmd_line):
    """
    msdn.microsoft.com/en-us/library/windows/desktop/bb762153(v=vs.85).aspx
    """
    cmd = shlex.split(cmd_line)[0]
    params = cmd_line.replace(cmd, '')
    hinstance = ctypes.windll.shell32.ShellExecuteW(None, 'runas', cmd, params, None, SW.HIDE)

    if hinstance <= 32:
        code = None
        # RuntimeError(ERROR(hinstance))
    else:
        code = hinstance

    return code


if __name__ == '__main__':
    is_user_admin = is_user_admin()
    print('userIsAdmin = %d' % (is_user_admin))
    if not is_user_admin:
        run_as_admin([sys.executable] + sys.argv, wait=True)



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Environment creation, import, deletion dialogs."""

from .backup_dialogs import *  # noqa: F403
from .clone_dialogs import *  # noqa: F403
from .conflict_dialogs import *  # noqa: F403
from .create_dialogs import *  # noqa: F403
from .import_dialogs import *  # noqa: F403
from .multiaccount_dialogs import *  # noqa: F403
from .remove_dialogs import *  # noqa: F403



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Classes used in CSS styling of login dialogs."""

__all__ = [
    'LabelMainLoginTitle', 'LabelMainLoginText', 'LabelMainLoginSubTitle', 'WidgetLoginInfoFrame',
    'WidgetLoginFormFrame', 'WidgetLoginCardsFrame', 'WidgetLoginCardsFrame', 'WidgetLoginPageContent',
    'SecondaryButton', 'WidgetLoginCard', 'WidgetNoticeFrame', 'LabelLoginLogo', 'WidgetHeaderFrame',
]

from anaconda_navigator.widgets import ButtonNormal, FrameBase
from anaconda_navigator.widgets.dialogs import LabelBase


class LabelMainLoginTitle(LabelBase):
    """Label used in CSS styling."""


class LabelMainLoginText(LabelBase):
    """Label used in CSS styling."""


class LabelMainLoginSubTitle(LabelBase):
    """Label used in CSS styling."""


class WidgetLoginInfoFrame(FrameBase):
    """Widget used in CSS styling."""


class WidgetLoginFormFrame(FrameBase):
    """Widget used in CSS styling."""


class WidgetLoginCardsFrame(FrameBase):
    """Widget used in CSS styling."""


class WidgetLoginPageContent(FrameBase):
    """Widget used in CSS styling."""


class SecondaryButton(ButtonNormal):
    """Label used in CSS styling."""


class WidgetLoginCard(FrameBase):
    """Widget used in CSS styling."""


class WidgetNoticeFrame(FrameBase):
    """Widget used in CSS styling."""


class LabelLoginLogo(LabelBase):
    """Label used in CSS styling."""


class WidgetHeaderFrame(FrameBase):
    """Widget used in CSS styling."""



# -*- coding: utf-8 -*-

"""
Utilities for reusable and conditional error processing.

This utilities allow converting this:

.. code-block:: python

    try:
        ...
    except ValueError as exception:
        result = fix_value_error(exception)
    except KeyError as exception:
        result = fix_key_error(exception)
    except requests.RequestsException as exception:
        if exception.response is not None:
            if (exception.response.status_code == 409) and (exception.response.json()['error']['code'] = 'some_exc'):
                result = fix_duplicate_environment(exception)
            elif (exception.response.status_code == 422):
                result = fix_unprocessable_entity(exception)
            else:
                result = fix_default(exception)
        else:
            result = fix_default(exception)
    except BaseException as exception:
        result = fix_default(exception)

to this:

.. code-block:: python

    handlers = HttpErrorHandlers()
    handlers.register_handler(BaseException, fix_default)
    handlers.register_handler(ValueError, fix_value_error)
    handlers.register_handler(KeyError, fix_key_error)
    handlers.register_http_handler(fix_duplicate_environment, 409, 'some_exc')
    handlers.register_http_handler(fix_unprocessable_entity, 422)

    try:
        ...
    except BaseException as exception:
        result = handlers.handle(exception)

.. rubric:: Handlers

Handlers are used to process exceptions. Each handler is associated with particular exception and exception context.

If you have multiple handlers for inherited exceptions - the closest one to the exception will be selected. E.g. you
have handlers for :code:`KeyError`, :code:`Exception` and :code:`BaseException`. For handling :code:`LookupError` will
be used handler for :code:`Exception`. For :code:`KeyboardInterrupt` - handler for :code:`BaseException`. And for any
"direct hit" - corresponding handler.

Exception context - is additional sequence of details, which can be used to select a more specific handlers. E.g. for
:exc:`~requests.RequestException` this context have next form:
:code:`[response.status_code, response.json()['error']['code']]`. If the :code:`response.json()['error']['code']` is
unavailable - this part of context is omitted.

Contexts are matched in a "longest prefix" manner. E.g. you have handlers for the same error, but with contexts
:code:`['a', 'b', 'c']`, :code:`['a', 'b', 'd']`, :code:`['a', 'b']`, :code:`['a', 'e']`. If you have exception with a
context :code:`['a', 'b', 'c', 'x']` - handler for context :code:`['a', 'b', 'c']` will be used. For exception with a
context :code:`['a', 'b']` - handler for context :code:`['a', 'b']` will be used. For exception with an empty context -
none of the above will be used.

This exception contexts are detected for each exception separately with exception parsers (see
:meth:`ErrorHandlers.register_parser`).

Functions to handle exceptions must have next format:

.. code-block:: python

    def handler(exception: BaseException) -> typing.Any:
        ...

Result value of the handler will be returned through the :meth:`ErrorHandlers.handle`.
"""

__all__ = ['ErrorHandlers', 'HttpErrorHandlers']

import typing
import requests

TConException = typing.TypeVar('TConException', bound=BaseException)

if typing.TYPE_CHECKING:
    import typing_extensions

    class Handler(typing_extensions.Protocol[TConException]):
        """Common protocol for error handling functions."""

        def __call__(self, exception: TConException) -> typing.Any:
            """
            Execute handler.

            :param exception: Exception to handle.
            :return: Result of the exception handling.
            """

    class Parser(typing_extensions.Protocol[TConException]):
        """Common protocol for context parsers for exceptions."""

        def __call__(self, exception: TConException) -> typing.Tuple[typing.Any, ...]:
            """
            Prepare exception context.

            :param exception: Exception, for which context must be prepared.
            :return: Context for the exception.
            """


class ErrorHandler(typing.NamedTuple):
    """
    Internal description of the error handler.

    .. py:attribute:: exception_type

        Type of the exception, that can be handled bu this handler.

    .. py:attribute:: context

        Context prefix required to execute this handler.

    .. py:attribute:: handler

        Function to handle an exception.
    """

    exception_type: typing.Type[BaseException]
    context: typing.Tuple[typing.Any, ...]
    handler: 'Handler'


class ErrorParser(typing.NamedTuple):
    """
    Internal description of the error parser.

    .. py:property:: exception_type

        Type of the exception, that can be handled bu this parser.

    .. py:property:: parser

        Function to prepare context for exception.
    """

    exception_type: typing.Type[BaseException]
    parser: 'Parser'


class ErrorHandlers:
    """Collection of error handlers."""

    __slots__ = ('__handlers', '__parsers')

    def __init__(self) -> None:
        """Initialize new :class:`~ErrorHandlers` instance."""
        self.__handlers: 'typing_extensions.Final[typing.List[ErrorHandler]]' = []
        self.__parsers: 'typing_extensions.Final[typing.List[ErrorParser]]' = []

    def handle(self, exception: BaseException) -> typing.Any:
        """
        Find handler for the `exception` and execute it.

        :param exception: Exception to handle.
        :return: Whatever handler returns.
        """
        context: typing.Tuple[typing.Any, ...] = ()

        parser: ErrorParser
        for parser in self.__parsers:
            if isinstance(exception, parser.exception_type):
                context = parser.parser(exception)
                break

        handler: ErrorHandler
        for handler in self.__handlers:
            if isinstance(exception, handler.exception_type) and (handler.context == context[:len(handler.context)]):
                return handler.handler(exception)

        return None

    def register_handler(
            self,
            exception_type: typing.Type[TConException],
            handler: 'Handler[TConException]',
            *context: typing.Any,
    ) -> None:
        """
        Add new exception handler.

        :param exception_type: Exception, which may be handled.
        :param handler: Handler function.
        :param context: Required context prefix.
        """
        index: int = 0
        while index < len(self.__handlers):
            current: ErrorHandler = self.__handlers[index]
            if issubclass(exception_type, current.exception_type):
                if current.exception_type != exception_type:
                    break
                if len(current.context) <= len(context):
                    break
            index += 1

        self.__handlers.insert(index, ErrorHandler(exception_type=exception_type, context=context, handler=handler))

    def register_parser(self, exception_type: typing.Type[TConException], parser: 'Parser[TConException]') -> None:
        """
        Add new exception parser.

        :param exception_type: Exceptions, which may be parsed.
        :param parser: Context parsing function.
        """
        index: int = 0
        while index < len(self.__parsers):
            if issubclass(exception_type, self.__parsers[index].exception_type):
                break
            index += 1

        self.__parsers.insert(index, ErrorParser(exception_type=exception_type, parser=parser))


class HttpErrorHandlers(ErrorHandlers):
    """Collection of error handlers, which is prepared to parse HTTP requests from NucleusAPI."""

    def __init__(self) -> None:
        """Initialize new :class:`~HttpErrorHandlers` instance."""
        super().__init__()
        self.register_parser(requests.RequestException, self.parse_http)

    @staticmethod
    def parse_http(exception: requests.RequestException) -> typing.Tuple[typing.Any, ...]:
        """Retrieve context for :class:`~requests.RequestException`."""
        result: typing.List[typing.Any] = []

        if exception.response is not None:
            result.append(exception.response.status_code)
            try:
                result.append(exception.response.json()['error']['code'])
            except (ValueError, TypeError, KeyError):
                pass

        return tuple(result)

    def register_http_handler(self, handler: 'Handler[requests.RequestException]', *context: typing.Any) -> None:
        """
        Add new exception handler for :class:`~requests.RequestException`.

        Interface is similar to :meth:`~ErrorHandlers.register_handler`, except missing `exception_type` argument.
        """
        self.register_handler(requests.RequestException, handler, *context)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Application start."""

# yapf: disable

# Standard library imports
import os
import signal
import sys

# Third party imports
from qtpy import PYQT5
from qtpy.QtCore import QCoreApplication, QEvent, QObject, Qt
from qtpy.QtGui import QIcon

# Local imports
from anaconda_navigator.config import (CONF, LINUX, LOCKFILE, MAC, PIDFILE,
                                       UBUNTU)
from anaconda_navigator.external import filelock
from anaconda_navigator.static import images
from anaconda_navigator.static.fonts import load_fonts
from anaconda_navigator.utils import misc
from anaconda_navigator.utils.logs import clean_logs, setup_logger
from anaconda_navigator.utils.qthelpers import qapplication
from anaconda_navigator.widgets.dialogs import MessageBoxInformation
from anaconda_navigator.widgets.dialogs.splash import SplashScreen
from anaconda_navigator.widgets.main_window import MainWindow


# yapf: enable

# For retina displays on qt5
if CONF.get('main', 'enable_high_dpi_scaling'):
    if hasattr(Qt, 'AA_UseHighDpiPixmaps'):
        QCoreApplication.setAttribute(Qt.AA_UseHighDpiPixmaps)

    if hasattr(Qt, 'AA_EnableHighDpiScaling'):
        QCoreApplication.setAttribute(Qt.AA_EnableHighDpiScaling)


def except_hook(cls, exception, traceback):
    """Custom except hook to avoid crashes on PyQt5."""
    sys.__excepthook__(cls, exception, traceback)


def set_application_icon():
    """Set application icon."""
    global app
    if LINUX and UBUNTU:
        app_icon = QIcon(images.ANACONDA_LOGO_WHITE)
    else:
        app_icon = QIcon(images.ANACONDA_LOGO)
    app.setWindowIcon(app_icon)


def run_app(splash):
    """Create and show Navigator's main window."""
    window = MainWindow(splash=splash)
    # window.setup()
    return window


class EventEater(QObject):
    """Event filter for application state."""
    def __init__(self, app):
        """Event filter for application state."""
        super(EventEater, self).__init__()
        self.app = app

    def eventFilter(self, ob, event):
        """Qt override."""
        if (event.type() == QEvent.ApplicationActivate and MAC and self.app.window.setup_ready):
            self.app.window.show()
            if self.app.window.isMaximized():
                self.app.window.showMaximized()
            elif self.app.window.isFullScreen():
                self.app.window.showFullScreen()
            else:
                self.app.window.showNormal()
            return True

        return super(EventEater, self).eventFilter(ob, event)


def start_app(options):  # pragma: no cover
    """Main application entry point."""
    # Setup logger
    setup_logger(options.log_level)

    # Monkey patching sys.excepthook to avoid crashes in PyQt 5.5+
    if PYQT5:
        sys.excepthook = except_hook

    # Clean old style logs
    clean_logs()

    global app
    app = qapplication(test_time=60)
    set_application_icon()
    load_fonts(app)
    signal.signal(signal.SIGINT, signal.SIG_DFL)

    # Check if running as root or with sudo on Unix
    if (MAC or LINUX) and os.environ.get('SUDO_UID', None) is not None:
        msgbox = MessageBoxInformation(
            title="Anaconda Navigator Information",
            text=("Anaconda Navigator cannot be run with root user "
                  "privileges.")
        )
        sys.exit(msgbox.exec_())

    # Create file lock
    lock = filelock.FileLock(LOCKFILE)
    try:
        load_pid = misc.load_pid()

        # This means a PSutil Access Denied error was raised
        if load_pid is False:
            msgbox = MessageBoxInformation(
                title="Anaconda Navigator Startup Error",
                text=(
                    "Navigator failed to start due to an incorrect shutdown. "
                    "<br><br>"
                    "We were unable to remove the pid & lock files. "
                    "Please manually remove the following files and restart "
                    "Anaconda Navigator:<br><ul>"
                    "<li><pre>{}</pre></li><li><pre>{}</pre></li></ul>"
                    "".format(LOCKFILE, PIDFILE)
                )
            )
            sys.exit(msgbox.exec_())
        elif load_pid is None:  # A stale lock might be around
            misc.remove_lock()

        with lock.acquire(timeout=3.0):  # timeout in seconds
            misc.save_pid()
            splash = SplashScreen()
            splash.show_message("Initializing...")
            window = run_app(splash)
            app.window = window
            event_eater = EventEater(app)
            app.installEventFilter(event_eater)
            sys.exit(app.exec_())
    except filelock.Timeout:
        msgbox = MessageBoxInformation(
            title="Anaconda Navigator Information",
            text=("There is an instance of "
                  "Anaconda Navigator already "
                  "running.")
        )
        sys.exit(msgbox.exec_())



# -*- coding: utf-8 -*-

"""Additional helper components used in Nucleus API integration."""

from .endpoints import *  # noqa: F403
from .json_streaming import *  # noqa: F403
from .sessions import *  # noqa: F403
from .workers import *  # noqa: F403



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Team Edition login dialogs."""

__all__ = ['TeamEditionSetDomainPage', 'TeamEditionLoginPage', 'TeamEditionAddChannelsPage']

import json
import typing
from qtpy.QtCore import Qt
from qtpy.QtGui import QPixmap
from qtpy.QtWidgets import QApplication, QHBoxLayout, QLabel, QVBoxLayout, QWidget
from anaconda_navigator.utils import get_domain_from_api_url
from anaconda_navigator.api.anaconda_api import AnacondaAPI
from anaconda_navigator.config import TEAM_EDITION_BRAND
from anaconda_navigator.utils.analytics import GATracker
from anaconda_navigator.widgets import ButtonPrimary
from anaconda_navigator.widgets.manager.channels import SelectableChannelsListTable
from anaconda_navigator.static.images import ANACONDA_TEAM_EDITION_LOGIN_LOGO
from . import styling
from .utils import TextContainer
from .base_dialogs import BasePage, BaseSettingPage, BaseLoginPage


TEAM_SET_DOMAIN_TEXT_CONTAINER = TextContainer(
    form_primary_text=(
        'Looks like this is the first time you are logging into Team Edition. Please set your Team Edition domain.'
    ),
    form_secondary_text='You only need to set this domain once. You can always change this in your Preferences.',
    form_input_label_text='Team Edition Domain',
    form_submit_button_text='Set Domain',
    info_frame_logo_path=ANACONDA_TEAM_EDITION_LOGIN_LOGO,
    info_frame_text='Login to configure Conda and Navigator to install packages from your Team Edition instance.',
)

TEAM_LOGIN_TEXT_CONTAINER = TextContainer(
    info_frame_logo_path=ANACONDA_TEAM_EDITION_LOGIN_LOGO,
    info_frame_text='Login to configure Conda and Navigator to install packages from your Team Edition instance.',
    message_box_error_text='The Team Edition API domain is not specified! Please, set in preferences.',
)


class TeamEditionSetDomainPage(BaseSettingPage):

    def __init__(self, parent: typing.Optional[QWidget] = None) -> None:
        super().__init__(AnacondaAPI(), TEAM_SET_DOMAIN_TEXT_CONTAINER, parent=parent)
        self.api_url_config_option = 'team_edition_api_url'
        self.input_line.setPlaceholderText('http(s)://example.com')
        self.button_apply.clicked.connect(self.set_domain)

    def check_text(self):
        text = self.input_line.text()
        domain = get_domain_from_api_url(text)
        self.input_line.setText('{}/api/system'.format(domain))
        is_valid = super(TeamEditionSetDomainPage, self).check_text()
        self.input_line.setText(domain)
        return is_valid


class TeamEditionLoginPage(BaseLoginPage):

    def __init__(self, parent: typing.Optional[QWidget] = None) -> None:
        super().__init__(AnacondaAPI(), TEAM_LOGIN_TEXT_CONTAINER, parent=parent)
        self.api_url_config_option = 'team_edition_api_url'
        self.brand = TEAM_EDITION_BRAND

    def _finished(self, worker, output, error):
        """
        Callback for the login procedure after worker has finished.

        If success, sets the token, 'username' attribute to parent widget
        and sends the accept signal.

        Otherwise, outputs error messages.
        """
        try:
            response = json.loads(output)

            if 'token' in response and 'refresh_token' in response:
                self.token = output
                self.accept()
                te_channels = TeamEditionAddChannelsPage()
                te_channels.open()

            else:
                if 'message' in response and response['message'] == 'Unauthorized':
                    self.label_message.setText('<i>Invalid Credentials!</i>')
                elif 'message' in response:
                    self.label_message.setText('<i>{}</i>'.format(response['message']))

                self.label_message.setVisible(True)
                self._track_error(output)

        except (json.decoder.JSONDecodeError, TypeError):
            self.label_message.setText('<i>Unhandled error happened!</i>')
            self.label_message.setVisible(True)
            self._track_error(error)

        self.button_login.setDisabled(False)
        self.check_text()
        QApplication.restoreOverrideCursor()

    def _track_error(self, error):
        domain = self.api.client_domain()
        label = '{0}/{1}: {2}'.format(domain, self.username, str(error).lower())
        self.tracker.track_event('authenticate', 'login failed', label=label)
        self.text_password.setFocus()
        self.text_password.selectAll()


class TeamEditionAddChannelsPage(BasePage):

    def __init__(
            self,
            msg: str = 'Select default channels to be used',
            btn_add_msg: str = 'Add Channels',
            parent: typing.Optional[QWidget] = None,
    ) -> None:
        super().__init__(parent)

        self.api = AnacondaAPI()
        self.tracker = GATracker()

        QApplication.restoreOverrideCursor()
        pixmap = QPixmap(ANACONDA_TEAM_EDITION_LOGIN_LOGO)
        pixmap = pixmap.scaledToWidth(200, Qt.SmoothTransformation)
        label_icon = styling.LabelLoginLogo()
        label_icon.setPixmap(pixmap)
        label_icon.setAlignment(Qt.AlignLeft)
        self.label_information = QLabel(msg)

        rc_data = self.api._conda_api.load_rc() or dict()
        api_channels_data = self.api.get_channels()
        channels = rc_data.get('channels', [])
        default_channels = rc_data.get('default_channels', [])

        self.channels_table = SelectableChannelsListTable(
            self, table_data=api_channels_data, channels=channels, default_channels=default_channels
        )
        self.channels_table.setMaximumWidth(650)

        self.button_skip = styling.SecondaryButton('Skip')
        self.button_add = ButtonPrimary(btn_add_msg)

        buttons_layout = QHBoxLayout()
        buttons_layout.addWidget(self.button_skip)
        buttons_layout.addWidget(self.button_add)

        self.main_layout = QVBoxLayout()
        self.main_layout.addWidget(label_icon, Qt.AlignLeft)
        self.main_layout.addWidget(self.label_information, Qt.AlignRight)
        self.main_layout.addWidget(self.channels_table, Qt.AlignCenter)
        self.main_layout.addLayout(buttons_layout)

        self.setLayout(self.main_layout)

        self.button_skip.clicked.connect(self.skip)
        self.button_add.clicked.connect(self.add_channels)

    def skip(self):
        self.api.create_login_data()
        self.accept()

    def add_channels(self):
        default_channels, channels = self.channels_table.get_selected_channels()

        if not default_channels:
            self.label_information.setText(
                'At least one channel should be added to <b>default_channels</b>! Please add...'
            )
            return

        self.api.create_login_data(default_channels, channels)
        self.accept()



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Tests for Anaconda API functions."""

# yapf: disable

# Standard library imports
import datetime
from collections import OrderedDict

# Third party imports
import pytest
from unittest import mock

# Local imports
from anaconda_navigator.api.anaconda_api import _AnacondaAPI
from anaconda_navigator.utils.fixtures import tmpfolder


# yapf: enable

DEFAULT_TIMEOUT = 120000
tmpfolder

# --- Fixtures
# -----------------------------------------------------------------------------
@pytest.fixture
def api(tmpfolder):
    api = _AnacondaAPI()
    return api


# --- Tests
# -----------------------------------------------------------------------------


def test_conda_data(qtbot, api):
    """"""
    def output_ready(worker, output, error):
        assert error is None
        assert len(output)
        assert 'info' in output
        assert 'processed_info' in output
        assert 'packages' in output
        assert 'applications' in output

    worker = api.conda_data(prefix=api.ROOT_PREFIX)
    with qtbot.waitSignal(worker.sig_chain_finished, timeout=DEFAULT_TIMEOUT, raising=True):
        worker.sig_chain_finished.connect(output_ready)


def test_process_packages(api, qtbot):
    BLACKLIST = ['python', 'anaconda-navigator']

    def conda_data_ready(worker, output, error):
        packages = output['packages']
        worker_2 = api.process_packages(
            packages,
            prefix=api.ROOT_PREFIX,
            blacklist=BLACKLIST,
        )
        worker_2.check_packages = packages

        with qtbot.waitSignal(worker_2.sig_chain_finished, timeout=DEFAULT_TIMEOUT, raising=True):
            worker_2.sig_chain_finished.connect(process_packages_ready)

    def process_packages_ready(worker, output, error):
        assert error is None
        assert len(output)
        # See: https://github.com/ContinuumIO/navigator/issues/1244
        for package_name in BLACKLIST:
            assert package_name in worker.check_packages

    worker = api.conda_data(prefix=api.ROOT_PREFIX)
    with qtbot.waitSignal(worker.sig_chain_finished, timeout=DEFAULT_TIMEOUT, raising=True):
        worker.sig_chain_finished.connect(conda_data_ready)


def test_filter_environments(api):
    not_filtered_envs = OrderedDict()
    not_filtered_envs['some_env_path1'] = '_build_env'
    not_filtered_envs['some_env_path2'] = '__build_env'
    not_filtered_envs['some_env_path3'] = 'build_env'
    not_filtered_envs['some_env_path4'] = 'build_env_'
    not_filtered_envs['some_env_path5'] = 'build_env__'
    not_filtered_envs['some_env_path6'] = '__build_env2'
    not_filtered_envs['some_env_path7'] = '__build_env3__'

    filtered_envs = api.filter_environments(not_filtered_envs)

    assert len(filtered_envs) == 3
    assert filtered_envs['some_env_path3'] == 'build_env'
    assert filtered_envs['some_env_path4'] == 'build_env_'
    assert filtered_envs['some_env_path5'] == 'build_env__'


def test_process_unsafe_channels_default_condarc(api):
    channels = [
        'https://repo.anaconda.com/pkgs/main/osx-64', 'https://repo.anaconda.com/pkgs/main/noarch',
        'https://repo.anaconda.com/pkgs/r/osx-64', 'https://repo.anaconda.com/pkgs/r/noarch',
        'https://repo.anaconda.com/pkgs/msys2/osx-64', 'https://repo.anaconda.com/pkgs/msys2/noarch'
    ]

    unsafe_channels = [
        'defaults', 'https://repo.anaconda.com/pkgs/main', 'https://repo.anaconda.com/pkgs/r',
        'https://repo.anaconda.com/pkgs/msys2'
    ]

    processed_channels = api._process_unsafe_channels(channels, unsafe_channels)

    expected_processed_channels = [
        'https://repo.anaconda.com/pkgs/main/osx-64', 'https://repo.anaconda.com/pkgs/main/noarch',
        'https://repo.anaconda.com/pkgs/r/osx-64', 'https://repo.anaconda.com/pkgs/r/noarch',
        'https://repo.anaconda.com/pkgs/msys2/osx-64', 'https://repo.anaconda.com/pkgs/msys2/noarch'
    ]
    assert processed_channels == expected_processed_channels


def test_process_unsafe_channels_ate_private_channels(api):
    channels = [
        'https://repo.anaconda.com/pkgs/main/osx-64', 'https://repo.anaconda.com/pkgs/main/noarch',
        'https://repo.anaconda.com/pkgs/r/osx-64', 'https://repo.anaconda.com/pkgs/r/noarch',
        'https://repo.anaconda.com/pkgs/msys2/osx-64', 'https://repo.anaconda.com/pkgs/msys2/noarch',
        'https://ate.training.anaconda.com/t/<TOKEN>/api/repo/adefusco/osx-64',
        'https://ate.training.anaconda.com/t/<TOKEN>/api/repo/adefusco/noarch'
    ]
    unsafe_channels = [
        'defaults',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/adefusco',
        'https://repo.anaconda.com/pkgs/main', 'https://repo.anaconda.com/pkgs/r',
        'https://repo.anaconda.com/pkgs/msys2'
    ]

    processed_channels = api._process_unsafe_channels(channels, unsafe_channels)

    expected_processed_channels = [
        'https://repo.anaconda.com/pkgs/main/osx-64', 'https://repo.anaconda.com/pkgs/main/noarch',
        'https://repo.anaconda.com/pkgs/r/osx-64', 'https://repo.anaconda.com/pkgs/r/noarch',
        'https://repo.anaconda.com/pkgs/msys2/osx-64', 'https://repo.anaconda.com/pkgs/msys2/noarch',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/adefusco/osx-64',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/adefusco/noarch'
    ]
    assert processed_channels == expected_processed_channels


def test_process_unsafe_channels_ate_channels_in_channels(api):
    channels = [
        'https://ate.training.anaconda.com/t/<TOKEN>/api/repo/main/osx-64',
        'https://ate.training.anaconda.com/t/<TOKEN>/api/repo/main/noarch',
        'https://ate.training.anaconda.com/t/<TOKEN>/api/repo/r/osx-64',
        'https://ate.training.anaconda.com/t/<TOKEN>/api/repo/r/noarch',
        'https://ate.training.anaconda.com/t/<TOKEN>/api/repo/msys2/osx-64',
        'https://ate.training.anaconda.com/t/<TOKEN>/api/repo/msys2/noarch',
        'https://ate.training.anaconda.com/t/<TOKEN>/api/repo/adefusco/osx-64',
        'https://ate.training.anaconda.com/t/<TOKEN>/api/repo/adefusco/noarch'
    ]
    unsafe_channels = [
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/main',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/r',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/msys2',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/adefusco',
        'https://repo.anaconda.com/pkgs/main', 'https://repo.anaconda.com/pkgs/r',
        'https://repo.anaconda.com/pkgs/msys2'
    ]

    processed_channels = api._process_unsafe_channels(channels, unsafe_channels)

    expected_processed_channels = [
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/main/osx-64',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/main/noarch',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/r/osx-64',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/r/noarch',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/msys2/osx-64',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/msys2/noarch',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/adefusco/osx-64',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/adefusco/noarch'
    ]
    assert processed_channels == expected_processed_channels


def test_process_unsafe_channels_ate_channels_in_default_channels(api):
    channels = [
        'https://ate.training.anaconda.com/t/<TOKEN>/api/repo/main/osx-64',
        'https://ate.training.anaconda.com/t/<TOKEN>/api/repo/main/noarch',
        'https://ate.training.anaconda.com/t/<TOKEN>/api/repo/r/osx-64',
        'https://ate.training.anaconda.com/t/<TOKEN>/api/repo/r/noarch',
        'https://ate.training.anaconda.com/t/<TOKEN>/api/repo/msys2/osx-64',
        'https://ate.training.anaconda.com/t/<TOKEN>/api/repo/msys2/noarch',
        'https://ate.training.anaconda.com/t/<TOKEN>/api/repo/adefusco/osx-64',
        'https://ate.training.anaconda.com/t/<TOKEN>/api/repo/adefusco/noarch'
    ]
    unsafe_channels = [
        'defaults',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/adefusco',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/main',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/r',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/msys2'
    ]

    processed_channels = api._process_unsafe_channels(channels, unsafe_channels)

    expected_processed_channels = [
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/main/osx-64',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/main/noarch',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/r/osx-64',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/r/noarch',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/msys2/osx-64',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/msys2/noarch',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/adefusco/osx-64',
        'https://ate.training.anaconda.com/t/90e35a25a887f7bdc0db7f883347f84ca0db964f88af3620/api/repo/adefusco/noarch'
    ]
    assert processed_channels == expected_processed_channels


def test_process_unsafe_channels_ce_channels(api):
    channels = [
        'https://repo.anaconda.cloud/t/<TOKEN>/repo/main/osx-64',
        'https://repo.anaconda.cloud/t/<TOKEN>/repo/main/noarch',
        'https://repo.anaconda.cloud/t/<TOKEN>/repo/msys2/osx-64',
        'https://repo.anaconda.cloud/t/<TOKEN>/repo/msys2/noarch',
        'https://repo.anaconda.cloud/t/<TOKEN>/repo/r/osx-64', 'https://repo.anaconda.cloud/t/<TOKEN>/repo/r/noarch'
    ]
    unsafe_channels = [
        'https://repo.anaconda.cloud/t/fc96b40a762d3826b7d958114336549769188541017cc3ef/repo/main',
        'https://repo.anaconda.cloud/t/fc96b40a762d3826b7d958114336549769188541017cc3ef/repo/msys2',
        'https://repo.anaconda.cloud/t/fc96b40a762d3826b7d958114336549769188541017cc3ef/repo/r',
        'https://repo.anaconda.com/pkgs/main', 'https://repo.anaconda.com/pkgs/r',
        'https://repo.anaconda.com/pkgs/msys2'
    ]
    processed_channels = api._process_unsafe_channels(channels, unsafe_channels)

    expected_processed_channels = [
        'https://repo.anaconda.cloud/t/fc96b40a762d3826b7d958114336549769188541017cc3ef/repo/main/osx-64',
        'https://repo.anaconda.cloud/t/fc96b40a762d3826b7d958114336549769188541017cc3ef/repo/main/noarch',
        'https://repo.anaconda.cloud/t/fc96b40a762d3826b7d958114336549769188541017cc3ef/repo/msys2/osx-64',
        'https://repo.anaconda.cloud/t/fc96b40a762d3826b7d958114336549769188541017cc3ef/repo/msys2/noarch',
        'https://repo.anaconda.cloud/t/fc96b40a762d3826b7d958114336549769188541017cc3ef/repo/r/osx-64',
        'https://repo.anaconda.cloud/t/fc96b40a762d3826b7d958114336549769188541017cc3ef/repo/r/noarch'
    ]

    assert processed_channels == expected_processed_channels


def test_setup_commercial_edition_default_channels(api):
    config_default_channels_mock = mock.patch('anaconda_navigator.api.anaconda_api.configure_default_channels').start()

    with mock.patch.object(api._conda_api, 'load_rc'):
        api._conda_api.load_rc.return_value = {'default_channels': []}
        api.setup_commercial_edition_default_channels()
        config_default_channels_mock.assert_called()

        config_default_channels_mock.reset_mock()

        api._conda_api.load_rc.return_value = {
            'default_channels': [api.config.get('main', 'commercial_edition_url') + "/repo/test/path"]
        }
        api.setup_commercial_edition_default_channels()
        config_default_channels_mock.assert_not_called()


def test_detect_commercial_edition_login(api):
    commercial_edition_base_url = api.config.get('main', 'commercial_edition_url')
    token_list_mock = mock.patch('anaconda_navigator.api.anaconda_api.token_list').start()

    with mock.patch.object(api._conda_api, 'load_rc'), \
            mock.patch.object(api, 'setup_commercial_edition_default_channels'), mock.patch.object(api.config, 'set'):

        token_list_mock.return_value = {}
        api.detect_commercial_edition_login()
        api.setup_commercial_edition_default_channels.assert_not_called()
        api.config.set.assert_not_called()

        api.setup_commercial_edition_default_channels.reset_mock()
        api.config.set.reset_mock()

        token_list_mock.return_value = {commercial_edition_base_url: 'secret_ce_token'}
        api.detect_commercial_edition_login()
        api.setup_commercial_edition_default_channels.assert_called()
        api.config.set.assert_has_calls(
            (mock.call('main', 'logged_brand', 'Commercial Edition'),
             mock.call('main', 'logged_api_url', commercial_edition_base_url)),
            any_order=True
        )
